import os

import setuptools


def _get_long_description():
    path = os.path.join(os.path.dirname(__file__), "README.rst")
    with open(path, "rb") as file:
        contents = file.read()
    return contents.decode("utf-8")


setuptools.setup(
    name="paka.cmark",
    description=(
        "Very lightweight CFFI-based Python bindings to cmark library"
        " (CommonMark implementation in C)."),
    long_description=_get_long_description(),
    version="1.10.0",
    packages=setuptools.find_packages(),
    setup_requires=["cffi>=1.0.0"],
    install_requires=["cffi>=1.0.0"],
    extras_require={"testing": []},
    cffi_modules=["paka/cmark/build_cmark.py:ffibuilder"],
    include_package_data=True,
    namespace_packages=["paka"],
    zip_safe=False,
    url="https://github.com/PavloKapyshin/paka.cmark",
    keywords="commonmark cmark c ffi",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy"],
    license="BSD",
    author="Pavlo Kapyshin",
    author_email="i@93z.org")



import os
import glob
import functools

from cffi import FFI


# Absolute path of paka/cmark dir.
CURRENT_PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))

# Absolute path of dir containing paka dir.
ROOT_DIR = os.path.normpath(os.path.join(CURRENT_PACKAGE_DIR, "../../"))

# Name of cmark "C sources" dir (with C sources, headers, etc.).
CMARK_SRC_DIR_NAME = "cmark_src"

# Absolute path of cmark "C sources" dir.
CMARK_SRC_DIR_PATH = os.path.join(CURRENT_PACKAGE_DIR, CMARK_SRC_DIR_NAME)

# Contents of cmark.h.
with open(os.path.join(CMARK_SRC_DIR_PATH, "cmark.h"), "rb") as file:
    CMARK_HEADER = file.read().decode("utf-8")
CMARK_HEADER = CMARK_HEADER.encode("ascii", "replace").decode("ascii")


_relativize = functools.partial(os.path.relpath, start=ROOT_DIR)
_relativize_paths = lambda paths: list(map(_relativize, paths))


try:
    _glob_escape = glob.escape
except AttributeError:
    _glob_escape = lambda s: s


def _get_sources(exclude):
    exclude = set(exclude)
    def _get_sources_paths():
        for path in glob.iglob(
                os.path.join(_glob_escape(CMARK_SRC_DIR_PATH), "*.c")):
            filename = os.path.basename(path)
            if filename not in exclude:
                yield path
    return _relativize_paths(sorted(_get_sources_paths()))


ffibuilder = FFI()
ffibuilder.cdef("""
#define CMARK_OPT_DEFAULT ...
#define CMARK_OPT_NOBREAKS ...
#define CMARK_OPT_HARDBREAKS ...
#define CMARK_OPT_SAFE ...

const char *cmark_version_string();

char *cmark_markdown_to_html(const char *text, size_t len, int options);
""")
ffibuilder.set_source(
    "paka.cmark._cmark",
    CMARK_HEADER,
    sources=_get_sources(exclude=["main.c"]),
    include_dirs=_relativize_paths([CMARK_SRC_DIR_PATH]))


if __name__ == "__main__":
    ffibuilder.compile(verbose=True)
